local gamera = require "lib/gamera"
local bump = require "lib/bump"

local world = bump.newWorld(48)
local Map = require "map"
local player = require "player"
local gravityAccel = 3500

DEBUG = true
WINDOW_HEIGHT, WINDOW_WIDTH = love.window.getDimensions()

function love.load()
    love.graphics.setDefaultFilter("nearest", "nearest", 1)

    world = bump.newWorld(48)

    local width, height = 4000, 2000
    cam = gamera.new(0, 0, width, height)
    Map:init("_assets/stage", width, height)
    Map:initObjectLayer(world)
    cam:setWindow(0, 0, 800, 600)

    player:init()

    world:add(player, player.l, player.t, player.w, player.h)
end


function love.update(dt)
    -- Apply gravity
    player.vy = player.vy + gravityAccel * dt
    player:update(dt)

    -- Update player position
    player.onGround = false

    local future_l = player.l + player.vx * dt
    local future_t = player.t + player.vy * dt

    local next_l, next_t, cols, len = world:move(player, future_l, future_t)

    for i=1, len do
        local col = cols[i]
        player:changeVelocityByCollisionNormal(col.normal.x, col.normal.y, bounciness)
        player:checkIfOnGround(col.normal.y)
    end

    player.l, player.t = next_l, next_t

    Map:update(dt)
    cam:setPosition(player:getCenter())

    -- Teleport player back when falling off
    if player.t > 800 then
        world:update(player, 0, 0, player.w, player.h)
    end
end

function love.draw()
    cam:draw(function(l, t, w, h)
        Map:draw()
        player:draw()
        if DEBUG then player:drawHitbox() end
    end)

    if DEBUG then
        love.graphics.print("l: " .. player.l, 10, 10)
        love.graphics.print("t: " .. player.t, 10, 25)
        love.graphics.print("speed: " .. player.speed, 10, 40)
        love.graphics.print("runAccel: " .. player.runAccel, 10, 55)
        love.graphics.print("brakeAccel: " .. player.brakeAccel, 10, 70)
        love.graphics.print("jumpVelocity: " .. player.jumpVelocity, 10, 85)
        love.graphics.print("isJumpingOrFlying: " .. tostring(player.isJumpingOrFlying), 10, 100)
        love.graphics.print("jumpMeter: " .. player.jumpMeter, 10, 115)
        love.graphics.print("vx: " .. player.vx, 10, 130)
        love.graphics.print("vy: " .. player.vy, 10, 145)
        love.graphics.print("canJump: " .. tostring(player.canJump), 10, 160)
        love.graphics.print("onGround: " .. tostring(player.onGround), 10, 175)

        local statistics = ("fps: %d, mem: %dKB"):format(love.timer.getFPS(), collectgarbage("count"))
        love.graphics.printf(statistics, WINDOW_HEIGHT - 200, WINDOW_WIDTH - 40, 200, 'right')
    end
end

function love.resize(w, h)
    Map.map:resize(w, h)
end

function love.keypressed(key, code)
    if key == "w" then
        local vx, vy = player.vx, player.vy

        if not player.isJumpingOrFlying then
            vy = -player.jumpVelocity
            player.isJumpingOrFlying = true
        end

        player.vx, player.vy = vx, vy
    end

    if key == "escape" then
        love.event.quit()
    end
end

function love.keyreleased(key)
    if key == "w" then
        player.canJump = false
    end
end
